local AddName, AddonTable = ...
local L = AddonTable.L

------------------------------
--		Options Equates		--
------------------------------
local Both, Enhanced, Base, Damage, DamageTaken, Percentage, Rating, MainHand, OffHand, World, Realm = 3, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2
local SameLevel, BossLevel, Regen, Casting, Critical, CritDamage, Total, Max, Average, Auto, Chance, Reduction = 1, 2, 1, 2, 1, 2, 1, 1, 2, 3, 1, 2

----------------------------------
--		Text Return Formats		--
----------------------------------
local Double_Rating_Format = { "%.1f", "%.0f", "%.1f (%.0f)", }

------------------------------
--		Global Updater		--
------------------------------

AddonTable.armorValue, AddonTable.armorDebuff, AddonTable.Resilvalue, AddonTable.ohID = 0, 0, 0, 0
AddonTable.dodgeChance, AddonTable.parryChance, AddonTable.blockChance, AddonTable.avoid, AddonTable.defAdjust, AddonTable.avoidBase = 0, 0, 0, 0, 0, 0

function AddonTable.DefenseUpdate()

 	AddonTable.targetLevel = AddonTable.playerLevel + 3
	if UnitCanAttack("player", "target") then AddonTable.targetLevel = UnitLevel("target")
		if AddonTable.targetLevel <= 0 then AddonTable.targetLevel = AddonTable.playerLevel + 3 end
	end	

	-- Armor
	local base, effectiveArmor, armor, posBuff, negBuff = UnitArmor("player")
	AddonTable.armorValue = effectiveArmor
	AddonTable.armorDebuff = negBuff
	
	-- Resilience
	AddonTable.Resilvalue = GetCombatRatingBonus(15)
	
	-- Block, Parry, Dodge
	AddonTable.blockChance = GetBlockChance("player")
	AddonTable.parryChance = GetParryChance("player")
	AddonTable.dodgeChance = GetDodgeChance("player")
	
	-- Avoidance / Crushing
	local ohID = GetInventoryItemID("player", 17)
	AddonTable.avoidBlock = 0
	local classId, subId = 0, 0
	AddonTable.avoidBlock = 0
	
	if ohID then
		_,_,_,_,_,_, _, _, _, _, _, classId, subId = GetItemInfo(ohID)
	end
		
	if classId == 4 and subId == 6 then
		AddonTable.avoidBlock = AddonTable.blockChance
	end
				
	AddonTable.defAdjust = (5 + (AddonTable.totalDefense - (AddonTable.targetLevel * 5)) * 0.04)
	AddonTable.avoidBase = (AddonTable.defAdjust + AddonTable.dodgeChance + AddonTable.parryChance + AddonTable.blockChance)
	AddonTable.avoid = (AddonTable.defAdjust + AddonTable.dodgeChance + AddonTable.parryChance + AddonTable.avoidBlock)
	AddonTable.avoid = AddonTable.avoid + AddonTable.scorpidSting + AddonTable.insectSwarm + AddonTable.frigidDread	+ AddonTable.quickness
end

-- Defense
function AddonTable.FunctionList.DefenseStat(HUD, data, options, ...)

	local EB, skillStat, ratingStat = options.Total_Rating
	local statFormat = Double_Rating_Format[EB]
	local defenseDisplay = AddonTable.totalDefense
	local returnText, decimals
	local capColor = ""
	
	if options.Display_Basic then defenseDisplay = AddonTable.baseDefense end
	
	if defenseDisplay >= 540 then capColor = AddonTable.greenText end
	
	if options.Dec_Def_Def == 0 then decimals = "%.0f"
	elseif options.Dec_Def_Def == 1 then decimals = "%.1f"
	elseif options.Dec_Def_Def == 2 then decimals = "%.2f"
	else decimals = "%.3f" end	
	
	if AddonTable.Band(EB, Total) then returnText = capColor .. (decimals):format(defenseDisplay) end
	if AddonTable.Band(EB, Rating) then returnText = ("%.0f"):format(AddonTable.defenseRating) end
	if AddonTable.Band(EB, Both) then returnText = capColor .. (decimals):format(defenseDisplay) .. " |r(" .. ("%.0f"):format(AddonTable.defenseRating) .. ")" end		
	
	HUD:UpdateText(data, returnText)
end	
------------------------------------------------

-- Armor
function AddonTable.FunctionList.Armor(HUD, data, options, ...)

	local EB, enhancedStat, baseStat = options.Enhanced_Base
	local returnText
	local debuffColor = ""
	
	if AddonTable.armorDebuff < 0 then debuffColor = AddonTable.redText end	
	
	if AddonTable.Band(EB, Enhanced) then returnText = debuffColor .. ("%.0f"):format(AddonTable.armorValue) end
	if AddonTable.Band(EB, Base) then returnText = ("%.0f"):format(AddonTable.armorValue) end
	if AddonTable.Band(EB, Both) then returnText = debuffColor .. ("%.0f"):format(AddonTable.armorValue) .. "/|r" .. AddonTable.armorValue end	
	
	HUD:UpdateText(data, returnText)
end
------------------------------------------------
	
-- Resilience
function AddonTable.FunctionList.Resilience(HUD, data, options, ...)

	local EB, percentStat, ratingStat = options.Crit_Damage_Taken
	local returnText, decimals, dualdec
	
	local Resilience = GetCombatRatingBonus(15)
	local resiDamage = GetCombatRatingBonus(16)
	resiDamage = resiDamage + Resilience
	
	if options.Dec_Def_Res == 0 then decimals = "%.0f%%" ; dualdec = "%.0f"
	elseif options.Dec_Def_Res == 1 then decimals = "%.1f%%" ; dualdec = "%.1f"
	elseif options.Dec_Def_Res == 2 then decimals = "%.2f%%" ; dualdec = "%.2f"
	else decimals = "%.3f%%"; dualdec = "%.3f" end		
	
	if AddonTable.Band(EB, Critical) then returnText = (decimals):format(Resilience) end
	if AddonTable.Band(EB, CritDamage) then returnText = (decimals):format(resiDamage) end
	if AddonTable.Band(EB, Both) then returnText = (dualdec):format(Resilience) .. "/|r" .. (decimals):format(resiDamage) end	
	
	HUD:UpdateText(data, returnText)
end
------------------------------------------------

-- Mitigation
function AddonTable.FunctionList.Mitigation(HUD, data, options, ...)

	local EB, enhancedStat, baseStat = options.Enhanced_Base
	local returnText, decimals
	local debuffColor = ""
	local armorReduction = (AddonTable.armorValue/(AddonTable.armorValue + 16635)) * 100
	
	if AddonTable.targetLevel < 80 then armorReduction = (AddonTable.armorValue/(AddonTable.armorValue+400+85*((5.5 * AddonTable.targetLevel)-265.5))) * 100
	elseif AddonTable.targetLevel == 80 then armorReduction = AddonTable.armorValue/(AddonTable.armorValue + 15232.5) * 100 end	
	
	local mitBase = armorReduction
	
	if GetShapeshiftForm() == 2 and (AddonTable.classFilename == "WARRIOR") then armorReduction = armorReduction + 10
	elseif GetShapeshiftForm() == 2 and (AddonTable.classFilename == "DEATHKNIGHT") then armorReduction = armorReduction + 8 end
	
	armorReduction = armorReduction + AddonTable.blessingSanc + AddonTable.divineProt + AddonTable.rightFury + AddonTable.bladeBarrier + AddonTable.renewedHope + 
	AddonTable.runeInvicibility + AddonTable.ancestralHealing + AddonTable.inspiration + AddonTable.GuardedByLight + AddonTable.ShieldTemplar + AddonTable.shamanRage +
	AddonTable.MoltenSkin + AddonTable.PrismaticCloak
	
	if AddonTable.armorDebuff < 0 then debuffColor = AddonTable.redText end
	
	if armorReduction >= 75 then debuffColor = AddonTable.greenText
	elseif armorReduction < 0 then armorReduction = 0; debuffColor = AddonTable.redText end

	if options.Dec_Def_Mit == 0 then decimals = "%.0f%%"
	elseif options.Dec_Def_Mit == 1 then decimals = "%.1f%%"
	elseif options.Dec_Def_Mit == 2 then decimals = "%.2f%%"
	else decimals = "%.3f%%" end	
	
	if AddonTable.Band(EB, Enhanced) then returnText = debuffColor ..  (decimals):format(armorReduction) end
	if AddonTable.Band(EB, Base) then returnText = (decimals):format(mitBase) end
	if AddonTable.Band(EB, Both) then returnText = debuffColor ..  (decimals):format(armorReduction) .. " |r(" .. (decimals):format(mitBase) .. ")" end

	HUD:UpdateText(data, returnText)
end
------------------------------------------------

-- Hit reduction
function AddonTable.FunctionList.HitReduction(HUD, data, options, ...)

	local hitReduction = AddonTable.scorpidSting + AddonTable.insectSwarm + AddonTable.frigidDread	+ AddonTable.quickness + AddonTable.Resilvalue
	local decimals
	
	if options.Dec_Def_Hit == 0 then decimals = "%.0f%%"
	elseif options.Dec_Def_Hit == 1 then decimals = "%.1f%%"
	elseif options.Dec_Def_Hit == 2 then decimals = "%.2f%%"
	else decimals = "%.3f%%" end		
	
	HUD:UpdateText(data, (decimals):format(hitReduction))
end
------------------------------------------------

-- Crit Immunity
function AddonTable.FunctionList.CritReceived(HUD, data, options, ...)

	local EB, percentStat, ratingStat = options.Chance_Reduction
	local chanceGetCrit = 19
	local getCritLevel = (AddonTable.targetLevel - AddonTable.playerLevel) * 0.2 -- 0.2 / 0.4 / 0.6 
	local getCritDefense = (AddonTable.totalDefense) * 0.04
	local uncritCap = 5
	local critTakenCap = 5.6
	local debuffColor = ""
	local returnText, decimals, liveImmunity
	local resilCrit = GetDodgeBlockParryChanceFromDefense()
	
	chanceGetCrit = (uncritCap + getCritLevel) - resilCrit - AddonTable.survFittest - AddonTable.Resilvalue - AddonTable.SleightTalent - AddonTable.moltenArmor - AddonTable.metaCrit
	
	if chanceGetCrit < 0 then chanceGetCrit = 0; debuffColor = AddonTable.greenText
	elseif chanceGetCrit <= 1.5 then debuffColor = AddonTable.orangeText
	else debuffColor = AddonTable.redText end
	
	critTakenCap = critTakenCap - chanceGetCrit
	
	if options.Dec_Def_Crit == 0 then decimals = "%.0f%%"
	elseif options.Dec_Def_Crit == 1 then decimals = "%.1f%%"
	elseif options.Dec_Def_Crit == 2 then decimals = "%.2f%%"
	else decimals = "%.3f%%" end	

	if options.Display_Simulate then liveImmunity = " > " .. (decimals):format(AddonTable.critReceived)
	else liveImmunity = "" end		
	
	if AddonTable.Band(EB, Chance) then returnText = debuffColor .. (decimals):format(chanceGetCrit) .. "|r" .. liveImmunity end
	if AddonTable.Band(EB, Reduction) then returnText = (decimals):format(critTakenCap) .. liveImmunity end
	if AddonTable.Band(EB, Both) then returnText = debuffColor .. (decimals):format(chanceGetCrit) .. " |r(" .. (decimals):format(critTakenCap) .. ")" .. liveImmunity end		

	HUD:UpdateText(data, returnText)
end
------------------------------------------------

-- Dodge
function AddonTable.FunctionList.Dodge(HUD, data, options, ...)

	local EB, percentStat, ratingStat = options.Percent_Rating
	local dodgeRating =	GetCombatRating(CR_DODGE)
	local returnText, decimals, liveDodge
	
	if options.Dec_Def_Dodge == 0 then decimals = "%.0f%%"
	elseif options.Dec_Def_Dodge == 1 then decimals = "%.1f%%"
	elseif options.Dec_Def_Dodge == 2 then decimals = "%.2f%%"
	else decimals = "%.3f%%" end

	if options.Display_Simulate then liveDodge = " > " .. (decimals):format(AddonTable.totalDodge)
	else liveDodge = "" end		
	
	if AddonTable.Band(EB, Percentage) then returnText = (decimals):format(AddonTable.dodgeChance) .. liveDodge end
	if AddonTable.Band(EB, Rating) then returnText = dodgeRating .. liveDodge end
	if AddonTable.Band(EB, Both) then returnText = (decimals):format(AddonTable.dodgeChance) .. " (" .. dodgeRating .. ")" .. liveDodge end
	
	HUD:UpdateText(data, returnText)
end
------------------------------------------------

-- Parry
function AddonTable.FunctionList.Parry(HUD, data, options, ...)

	local EB, percentStat, ratingStat = options.Percent_Rating
	local parryRating =	GetCombatRating(CR_PARRY)
	local returnText, decimals, liveParry

	if options.Dec_Def_Parry == 0 then decimals = "%.0f%%"
	elseif options.Dec_Def_Parry == 1 then decimals = "%.1f%%"
	elseif options.Dec_Def_Parry == 2 then decimals = "%.2f%%"
	else decimals = "%.3f%%" end
	
	if options.Display_Simulate then liveParry = " > " .. (decimals):format(AddonTable.totalParry)
	else liveParry = "" end			
	
	if AddonTable.Band(EB, Percentage) then returnText = (decimals):format(AddonTable.parryChance) .. liveParry end
	if AddonTable.Band(EB, Rating) then returnText = parryRating .. liveParry end
	if AddonTable.Band(EB, Both) then returnText = (decimals):format(AddonTable.parryChance) .. " (" .. parryRating .. ")" .. liveParry end
	
	HUD:UpdateText(data, returnText)
end
------------------------------------------------

-- Block
function AddonTable.FunctionList.Block(HUD, data, options, ...)

	local EB, percentStat, ratingStat = options.Percent_Rating
	local blockRating =	GetCombatRating(CR_BLOCK)
	local returnText, decimals, liveBlock

	if options.Dec_Def_Block == 0 then decimals = "%.0f%%"
	elseif options.Dec_Def_Block == 1 then decimals = "%.1f%%"
	elseif options.Dec_Def_Block == 2 then decimals = "%.2f%%"
	else decimals = "%.3f%%" end
	
	if options.Display_Simulate then liveBlock = " > " .. (decimals):format(AddonTable.totalBlock)
	else liveBlock = "" end		
	
	if AddonTable.Band(EB, Percentage) then returnText = (decimals):format(AddonTable.blockChance) .. liveBlock end
	if AddonTable.Band(EB, Rating) then returnText = blockRating .. liveBlock end
	if AddonTable.Band(EB, Both) then returnText = (decimals):format(AddonTable.blockChance) .. " (" .. blockRating .. ")" .. liveBlock end
	
	HUD:UpdateText(data, returnText)
end
------------------------------------------------

-- Avoidance
function AddonTable.FunctionList.Avoidance(HUD, data, options, ...)
	
	local EB, enhancedStat, baseStat = options.Enhanced_Base
	local returnText, decimals
	local avoidColor = ""
	
	if 	AddonTable.avoid >= 102.4 then avoidColor = AddonTable.greenText
	else avoidColor = "" end
	
	if options.Dec_Def_Avoidance == 0 then decimals = "%.0f%%"
	elseif options.Dec_Def_Avoidance == 1 then decimals = "%.1f%%"
	elseif options.Dec_Def_Avoidance == 2 then decimals = "%.2f%%"
	else decimals = "%.3f%%" end
	
	if AddonTable.Band(EB, Enhanced) then returnText = avoidColor .. (decimals):format(AddonTable.avoid) end
	if AddonTable.Band(EB, Base) then returnText = avoidColor .. (decimals):format(AddonTable.avoidBase) end	
	if AddonTable.Band(EB, Both) then returnText = avoidColor .. (decimals):format(AddonTable.avoid) .. " |r(" .. avoidColor .. (decimals):format(AddonTable.avoidBase) .. ")" end	

	HUD:UpdateText(data, returnText)
end
------------------------------------------------

-- Crushing
function AddonTable.FunctionList.Crushing(HUD, data, options, ...)
	
	local EB, enhancedStat, baseStat = options.Enhanced_Base
	local returnText, decimals
	local crushColor = ""
	local targetCrushLevel = UnitLevel("target")
	
	if targetCrushLevel == 0 then targetCrushLevel = AddonTable.playerLevel + 4 end

	local crush = ((targetCrushLevel * 5) - AddonTable.totalDefense) * 2 - 15	

	if crush <= 0 then crushColor = AddonTable.greenText; crush = 0
	else crushColor = "" end
	
	if options.Dec_Def_Crush == 0 then decimals = "%.0f%%"
	elseif options.Dec_Def_Crush == 1 then decimals = "%.1f%%"
	elseif options.Dec_Def_Crush == 2 then decimals = "%.2f%%"
	else decimals = "%.3f%%" end	
	
	if AddonTable.Band(EB, Enhanced) then returnText = crushColor .. (decimals):format(crush) end
	if AddonTable.Band(EB, Base) then returnText = crushColor .. (decimals):format(crush) end	
	if AddonTable.Band(EB, Both) then returnText = crushColor .. (decimals):format(crush) .. " |r(" .. crushColor .. (decimals):format(crush) .. ")" end		

	HUD:UpdateText(data, returnText)
end
------------------------------------------------
